#!/usr/bin/env python
# -*- coding: euc-jp -*-


"""
 \file CORBA_SeqEx.py
 \brief CORBA utility template classes
 \date $Date: 2007/09/12 $
 \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara

 Copyright (C) 2006
     Task-intelligence Research Group,
     Intelligent Systems Research Institute,
     National Institute of
         Advanced Industrial Science and Technology (AIST), Japan
     All rights reserved.
"""

import threading
import OpenRTM


class ScopedLock:
	
	def __init__(self, mutex):
		self.mutex = mutex
		self.mutex.acquire()

	def __del__(self):
		self.mutex.release()


def for_each(seq, f):
	"""
	\if jp
	\brief CORBA sequence Ф functor ŬѤ
	
	CORBA sequence ƤǤФơͿ줿 functor ŬѤ롣
	functor  void functor(CORBA sequence ) ηȤɬפ롣
	
	\return ƤǤ Functor
	\param seq Functor ŬѤ CORBA sequence
	\param functor CORBA sequence Ǥ Functor
	
	\else
	\brief Apply the functor to all CORBA sequence elements
	
	Apply the given functor to the given CORBA sequence.
	functor should be void functor(CORBA sequence element).
	
	\return Functor that processed all CORBA sequence elements
	\param seq CORBA sequence to be applied the functor
	\param functor A functor to process CORBA sequence elements
	\endif
	"""   
	len_ = len(seq)

	for i in range(len_):
		f(seq[i])

	return f


def find(seq, f):
	"""   
	\if jp
	\brief CORBA sequence 椫 functor Ŭ礹ǤΥǥå֤
	
	CORBA sequence ƤǤФơͿ줿 functor ŬѤ
	functor  true ֤褦Υǥå֤
	functor  bool functor(const CORBA sequence ) ηȤꡢ
	Ŭ礹ǤФ true ֤ɬפ롣
	
	\return Functor Ŭ礹ǤΥǥå
	ĤʤȤ -1 ֤
	\param seq Functor ŬѤ CORBA sequence
	\param functor CORBA sequence Ǥ򸫤Ĥ Functor
	
	\else
	\brief Return the index of CORBA sequence element that functor matches 
    
	This operation applies the given functor to the given CORBA sequence,
	and returns the index of the sequence element that the functor matches.
	The functor should be bool functor(const CORBA sequence element) type,
	and it would return true, if the element matched the functor.
	
	\return The index of the element that functor matches.
	If no element found, it would return -1.
	\param seq CORBA sequence to be applied the functor
	\param functor A functor to process CORBA sequence elements
	\endif
	"""   
	len_ = len(seq)

	for i in range(len_):
		if f(seq[i]):
			return i
	return -1


def push_back(seq, elem):
	"""
	\if jp
	\brief CORBA sequence κǸǤɲä
	
	CORBA sequence κǸͿ줿Ǥɲä롣
	CORBA sequence ĹϼưŪ˳ĥ롣
	
	\param seq Ǥɲä CORBA sequence
	\param elem ɲä
	
	\else
	\brief Push the new element back to the CORBA sequence
	
	Add the given element to the last of CORBA sequence.
	The length of the CORBA sequence will be expanded automatically.
	
	\param seq CORBA sequence to be added a new element
	\param elem The new element to be added to the CORBA sequence
	\endif
	"""
	seq.append(elem)


def insert(seq, elem, index):
	"""
	\if jp
	\brief CORBA sequence Ǥ
	
	CORBA sequence  index ΰ֤Ǥä롣
	index  Ϳ줿CORBA sequence κ index 礭
	ǸǤȤƲä롣
	CORBA sequence ĹϼưŪ˳ĥ롣
	
	\param seq Ǥɲä CORBA sequence
	\param elem ɲä
	\param index Ǥɲä
	
	\else
	\brief Insert the element to the CORBA sequence
	
	Insert a new element in the given position to the CORBA sequence.
	If the given index is greater than the length of the sequence,
	the given element is pushed back to the last of the sequence.
	The length of the CORBA sequence will be expanded automatically.
	
	\param seq The CORBA sequence to be inserted a new element
	\param elem The new element to be inserted the sequence
	\param index The inserting position
	\endif
	"""
	len_ = len(seq)
	if index > len:
		seq.append(elem)
		return
	seq.insert(index, elem)


def front(seq):
	"""
	\if jp
	\brief CORBA sequence ƬǤ
	
	seq[0] Ʊ
	
	\param seq Ǥ CORBA sequence
	
	\else
	\brief Get the front element of the CORBA sequence
	
	This operation returns seq[0].
	
	\param seq The CORBA sequence to be get the element
	\endif
	"""
	return seq[0]


def back(seq):
	"""
	\if jp
	\brief CORBA sequence Ǥ
	
	seq[seq.length() - 1] Ʊ
	
	\param seq Ǥ CORBA sequence
	
	\else
	
	\brief Get the last element of the CORBA sequence
	
	This operation returns seq[seq.length() - 1].
	
	\param seq The CORBA sequence to be get the element
	\endif
	"""
	return seq[-1]


def erase(seq, index):
	"""
	\if jp
	\brief CORBA sequence λꤵ줿֤Ǥ
	
	ꤵ줿ǥåǤ롣
	줿Ǥϵͤ졢sequence Ĺ1롣
	
	\param seq Ǥ CORBA sequence
	\param index ǤΥǥå
	
	\else
	
	\brief Erase the element of the specified index
	
	This operation removes the element of the given index.
	The other elements are closed up around the hole.
	
	\param seq The CORBA sequence to be get the element
	\param index The index of the element to be removed
	\endif
	"""
	if index > len(seq):
		return

	del seq[index]


def clear(seq):
	"""
	\if jp
	\brief CORBA sequence Ǥ
	
	seq.length(0) Ʊ
	
	\else
	\brief Erase all the elements of the CORBA sequence
	
	same as seq.length(0).
	\endif
	"""
	del seq[0:]



""" CORBA sequence extention class """

class LockedStruct:
	
	def __init__(self):
		self.lock = threading.RLock()
		self.data = None

class SequenceEx:
	
	"""
	\if jp
	\class SequenceEx
	\brief CORBA::sequence ĥ饹
	
	Υ饹 CORBA  sequence ĥ std::vector Υ󥿡ե
	󶡤 (㤨 size(), max_size(), empty(), resize(), insert(),
	erase(), erase_if(), push_back(), pop_back(), find()).
	CORBA  sequence ѾƤ뤿ᡢCORBA  sequence 
	ڥ졼(like operator=(), maximum(), length(), operator[])
	ѲǽǤ롣

	\else
	\brief CORBA::sequence extention class
	
	This class extends CORBA sequence type, and provides std::vector like
	interfaces (like size(), max_size(), empty(), resize(), insert(),
	erase(), erase_if(), push_back(), pop_back(), find()).
	Since this class inherits CORBA sequence class, user can also use CORBA
	sequence interfaces (like operator=(), maximum(), length(), operator[]).
	\endif
	"""


	def __init__(self, _sq):
		"""
		\if jp
		
		\brief CorbaSequence Υԡ󥹥ȥ饯
		
		CorbaSequenceΥԡ󥹥ȥ饯
		Ϳ줿 CorbaSequence Ƥ򥳥ԡ롣
		
		\param _sq CorbaSequence Υԡ
		
		\else
		\brief Copy constructor from CorbaSequence
		
		This constructor copies sequence contents from given CorbaSequence
		to this object.
		
		\param _sq Copy source of CorbaSequence type
		\endif
		"""
		len_ = len(_sq)
		self._seq = []
		for i in range(len_):
			self._seq.append(_sq[i])
		self._mutex = threading.RLock()


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		self._seq = []


	def size(self):
		"""
		\if jp
		
		\brief 
		
		Υڥ졼ϥ󥹤Υ֤
		CorbaSequence::length() Ʊ
		\return 󥹤Υ
		
		\else
		\brief Get size of this sequence
		
		This operation returns the size of the sequence.
		This is same as CorbaSequence::length().
		\return The size of the sequence.
		\endif
		"""
		return len(self._seq)


	def max_size(self):
		"""
		\if jp
		\brief ǼǽʺΥ
		
		Υڥ졼ϥ󥹤θߤγǼǽʺΥ֤
		CorbaSequence::maximum() Ʊ
		\return 󥹤˳ǼǽʺΥ
		
		\else
		\brief Get current maximum size of this sequence
		
		This operation returns the current maximum size of the sequence.
		This is same as CorbaSequence::maximum().
		\return The maximum size of the sequence.
		\endif
		"""
		return len(self._seq)


	def empty(self):
		"""
		\if jp
		\brief 󥹤ɤĴ٤
		
		Υڥ졼ϥ󥹤ɤ bool ֤ͤ
		 0 ʤ trueǤʤ false ֤
		\return 󥹤ɤ bool 
		
		\else
		\brief Test whether the sequence is empty
		
		This operation returns bool value whether the sequence is empty.
		If the size of the sequence is 0, this operation returns true,
		and in other case this operation returns false.
		\return The bool value whether the sequence is empty.
		\endif
		"""
		if not self._seq:
			return False
		else:
			return True


	def resize(self, new_size, item):
		"""
		\if jp
		\brief 󥹤ꥵ
		
		Υڥ졼ϥ󥹤Ĺѹ롣
		ߤĹ礭ʥͿ줿硢 x ǡ
		˥Ȥ줿ʬ롣
		ߤĹ꾮Ϳ줿硢CorabSequence Ʊͤ
		;ʬʥ󥹤ǤϺ롣
		\param new_size 󥹤Υ
		\param itemĹʤäʬΥ󥹤
		
		\else
		\brief Resize the length of the sequence
		
		This operation resizes the length of the sequence.
		If longer length than current sequence length is given,
		newly allocated rooms will be assigned by element given by the argument.
		If shorter length than current sequence length is given,
		the excessive element of a sequence is deleted like behavior of
		CorabSequence
		\param new_size The new size of the sequence
		\param item   Sequence element to be assigned to new rooms.
		\endif
		"""
		guard = ScopedLock(self._mutex)
		len_ = len(self._seq)
		if new_size > len_:
			self._seq = []
			for i in range(len_):
				self._seq.append(item)
		elif new_size < len_:
			del self._seq[new_size:]


	def insert(self, position, item):
		"""
		\if jp
		\brief 󥹤Ǥ
		
		Υڥ졼ϥ󥹤Ǥ롣
		\param position Ǥ
		\param item륷󥹤
		
		\else
		\brief Insert a new item to the sequence
		
		This operation inserts a new item to the sequence.
		\param position The position of new inserted item.
		\param item   Sequence element to be inserted.
		\endif
		"""
		guard = ScopedLock(self._mutex)
		len_ = len(self._seq)
		if position > len_:
			raise

		self._seq.insert(position, item)


	def erase(self, position):
		"""
		\if jp
		\brief 󥹤Ǥ
		
		Υڥ졼ϥ󥹤Ǥ
		\param position 륷Ǥξ
		
		\else
		\brief Erase an item of the sequence
		
		This operation erases an item from the sequence.
		\param position The position of erased item.
		\endif
		"""
		guard = ScopedLock(self._mutex)
		len_ = len(self._seq)
		if position > (len_ - 1):
			raise

		erased = self._seq[position]
		del self._seq[position]
		return erased


	def erase_if(self, f):
		"""
		\if jp
		\brief 󥹤ǤҸΤäƺ
		
		Υڥ졼ϽҸȤͿ줿ؿ֥Ȥ
		郎ΤȤΥ󥹤Ǥ롣
		\param f 륷󥹤ꤹѸ
		
		\else
		\brief Erase an item according to the given predicate
		
		This operation erases an item according to the given predicate.
		\param f The predicate functor to decide deletion.
		\endif
		"""
		guard = ScopedLock(self._mutex)
		len_ = len(self._seq)
		for i in range(len_):
			if f(self._seq[i]):
				return self.erase(i)
		raise


	def push_back(self, item):
		"""
		\if jp
		\brief ǤǸɲä
		
		Υڥ졼Ϳ줿Ǥ򥷡󥹤κǸɲä롣
		\param item ɲä뤹륪֥
		
		\else
		\brief Append an item to the end of the sequence.
		
		This operation push back an item to the of the sequence.
		\param item The object to be added to the end of the sequnce.
		\endif
		"""
		guard = ScopedLock(self._mutex)
		self._seq.append(item)


	def pop_back(self):
		guard = ScopedLock(self._mutex)
		del self._seq[-1]


	def find(self, f):
		guard = ScopedLock(self._mutex)
		len_ = len(self._seq)
		for i in range(len_):
			if f(self._seq[i]):
				return self._seq[i]
		raise
