#!/usr/bin/env python
# -*- Python -*-


"""
  \file CorbaConsumer.py
  \brief CORBA Consumer class
  \date $Date: 2007/09/20 $
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""

from omniORB import CORBA

class CorbaConsumerBase:
	
	"""
	\if jp
	\class ConsumerBase
	\brief ֥ȥե󥹤ݻץ졼ۥ쥯饹
	\else
	\class ConsumerBase
	\brief Placeholder base class to hold remote object reference.
	\endif
	"""


	def __init__(self, consumer=None):
		"""
		\if jp
		\brief 󥹥ȥ饯
		\param consumer(CorbaConsumerBase&) CorbaConsumerBase֥
		\else
		\brief Consructor
		\param consumer(CorbaConsumerBase&) CorbaConsumerBase object
		\endif
		"""
		if consumer:
			self._objref = consumer._objref
		else:
			self._objref = None


	def equal(self, consumer):
		self._objref = consumer._objref
		return self


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		pass


	def setObject(self, obj):
		"""
		\if jp
		\brief CORBA֥Ȥ򥻥åȤ
		
		Ϳ줿֥ȥե󥹤ϡConsumerBase ֥
		ݻ롣 
		
		\param obj CORBA ֥ȤΥե
		\return obj  nil ե󥹤ξ false ֤
		\else
		\brief Set CORBA Object
		
		The given CORBA Object is held.
		\param obj Object reference of CORBA object
		\return If obj is nil reference, it returns false.
		\endif
		"""
		if CORBA.is_nil(obj):
			return False

		self._objref = obj
		return True


	def getObject(self):
		"""
		\if jp
		\brief CORBA֥Ȥ
		
		setObject(obj)ˤͿ줿֥ȥե󥹤֤
		\return obj CORBA ֥ȤΥե
		\else
		\brief Set CORBA Object
		
		return The CORBA Object reference that given by setObject(obj)
		\return obj Object reference of CORBA object
		\endif
		"""
		return self._objref


	def releaseObject(self):
		self._objref = CORBA.Object._nil


class CorbaConsumer(CorbaConsumerBase):
	
	"""
	\if jp
	\class Consumer
	\brief ֥ȥե󥹤ݻץ졼ۥƥץ졼ȥ饹
	
	interfaceTypeͿ줿Υ֥Ȥݻ롣
	֥ȤåȤ줿ȤˡͿ줿 narrow Τǡ
	_ptr() Ǽե󥹤ϡnarrow ѤߤΥե󥹤Ǥ롣
	\param interfaceType Υۥݻ륪֥Ȥη
	\param consumer Υ饹Υ֥
	\else
	\class Consumer
	\brief Placeholder template class to hold remote object reference.
	
	This class holds a type of object that given by interfaceType parameter.
	\endif
	"""

	def __init__(self, interfaceType=None, consumer=None):
		"""
		\if jp
		\brief 󥹥ȥ饯
		\param interfaceType idlեƤ륤󥿡ե
		\param consumer(CorbaConsumer&) CorbaConsumer֥
		\else
		\brief Consructor
		\param interfaceType Type of interface defined in your xxx.idl
		\param consumer(CorbaConsumer&) CorbaConsumer object
		\endif
		"""
		if interfaceType:
			self._interfaceType = interfaceType
		else:
			self._interfaceType = None

		if consumer:
			CorbaConsumerBase.__init__(self, consumer)
			self._var = consumer._var
		else:
			CorbaConsumerBase.__init__(self)
			self._var = None


	def equal(self, consumer):
		self._var = consumer._var


	def __del__(self):
		"""
		\if jp
		\brief ǥȥ饯
		\else
		\brief Destructor
		\endif
		"""
		pass


	def setObject(self, obj):
		"""
		\if jp
		\brief ֥Ȥ򥻥åȤ
		ConsumerBase ΥС饤ɡ CorbaConsumerBase._objref˥֥
		򥻥åȤȤȤˡinterfaceType narrow ֥Ȥ
		ѿݻ롣
		\param obj CORBA Objecct
		\return True or False
		\else
		\brief Set Object
		Override function of ConsumerBase. This operation set an Object to 
		self._objref in the CorbaConsumerBase class, and this object is narrowed to
		given interfaceType parameter and stored in the member variable.
		\param obj CORBA Objecct
		\return True or False
		\endif
		"""
		if CorbaConsumerBase.setObject(self, obj):
			if self._interfaceType:
				self._var = obj._narrow(self._interfaceType)
			else:
				self._var = self._objref
			if not CORBA.is_nil(self._var):
				return True
		return False


	def _ptr(self):
		"""
		\if jp
		\brief ֥ȤnarrowѤߤΥ֥ȥե󥹤
		
		֥ȤΥե󥹤롣
		֥ȥե󥹤ѤˤϡsetObject() ǥåȺѤߤ
		ʤФʤʤ
		֥ȤåȤƤʤСnil ֥ȥե󥹤
		֤롣
		\return interfaceTypenarrowѤߤΥ֥ȤΥե
		\else
		\brief Get Object reference narrowed as interfaceType
		
		This operation returns object reference narrowed as interfaceType.
		To use the returned object reference, reference have to be set by
		setObject().
		If object is not set, this operation returns nil object reference.
		\return The object reference narrowed as interfaceType
		\endif
		"""
		return self._var


	def releaseObject(self):
		CorbaConsumerBase.releaseObject(self)
		self._var = CORBA.Object._nil
