#!/usr/bin/env python
# -*- Python -*-


"""
 \file CorbaNaming.py
 \brief CORBA naming service helper class
 \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara

 Copyright (C) 2006
     Noriaki Ando
     Task-intelligence Research Group,
     Intelligent Systems Research Institute,
     National Institute of
         Advanced Industrial Science and Technology (AIST), Japan
     All rights reserved.
"""

import omniORB.CORBA as CORBA
import CosNaming
import string

class CorbaNaming:
	
	"""
	\if jp
	\class CorbaNaming
	\brief CORBA Naming Service إѡ饹
	
	Υ饹ϡCosNaming::NamingContext Фåѡ饹Ǥ롣
	CosNaming::NamingContext ĥڥ졼ȤۤƱǽ
	ڥ졼󶡤ȤȤˡ͡ॳݡͥ CosNaming::Name
	ʸˤ̾ɽդ륪ڥ졼󶡤롣
	
	֥Ȥ뤤ľ CORBA ͡ॵФ³
	ʸ塢Υ͡ॵФΥ롼ȥƥȤФƼΥڥ졼
	롣
	ؤΥ͡ߥ󥰥ƥȤκ䥪֥ȤΥХɤˤơ
	ΥƥȤ¸ߤʤǤ⡢Ū˥ƥȤХ
	ŪΥƥȤ䥪֥ȤΥХɤԤȤǤ롣
	
	\else
	\class CorbaNaming
	\brief CORBA Naming Service helper class
	
	This class is a wrapper class of CosNaming::NamingContext.
	Almost the same operations which CosNaming::NamingContext has are
	provided, and some operation allows string naming representation of
	context and object instead of CosNaming::Name.
	
	The object of the class would connect to a CORBA naming server at
	the instantiation or immediately after instantiation.
	After that the object invokes operations to the root context of it.
	This class realizes forced binding to deep NamingContext, without binding
	intermediate NamingContexts explicitly.
	\endif
	"""


	def __init__(self, orb, name_server=None):
		"""
		 \if jp
		 \brief
		 \param
		 \param

		 \else
		 \brief
		 \param
		 \param
		 \endif
		"""
		self._orb = orb
		self._nameServer = ""
		self._rootContext = CosNaming.NamingContext._nil
		self._blLength = 100

		if name_server != None:
			self._nameServer = "corbaloc::" + name_server + "/NameService"
			try:
				obj = orb.string_to_object(self._nameServer)
				self._rootContext = obj._narrow(CosNaming.NamingContext)
				if CORBA.is_nil(self._rootContext):
					print "CorbaNaming: Failed to narrow the root naming context."

			except CORBA.ORB.InvalidName:
				print "Service required is invalid [does not exist]."

		return
	

	def __del__(self):
		return


	def init(self, name_server):
		self._nameServer = "corbaloc::" + name_server + "/NameService"
		obj = self._orb.string_to_object(self._nameServer)
		self._rootContext = obj._narrow(CosNaming.NamingContext)
		if CORBA.is_nil(self._rootContext):
			raise MemoryError

		return


	def bind(self, name_list, obj, force=None):
		"""
		\if jp
		
		\brief Object  bind 
		
		CosNaming::bind() ȤۤƱƯ򤹤뤬Ϳ줿͡ॵФ
		롼ȥƥȤФbind()ƤӽФۤʤ롣
		
		Name <name>  Object <obj>  NamingContext ˥Хɤ롣
		c_n  n ܤ NameComponent 򤢤魯Ȥȡ
		name  n Ĥ NameComponent ȤʲΤ褦˰롣
		
		cxt->bind(<c_1, c_2, ... c_n>, obj) ϰʲƱǤ롣
		cxt->resolve(<c_1, ... c_(n-1)>)->bind(<c_n>, obj)
		
		ʤ1ܤn-1ܤΥƥȤ褷n-1ܤΥƥ
		 name <n> Ȥơobj  bind 롣
		̾˻ä <c_1, ... c_(n-1)>  NemingContext ϡ
		bindContext()  rebindContext() Ǵ˥ХɺѤߤǤʤФʤʤ
		⤷ <c_1, ... c_(n-1)>  NamingContext ¸ߤʤˤϡ
		NotFound 㳰ȯ롣
		
		Хɥե饰 force  true λϡ<c_1, ... c_(n-1)>
		¸ߤʤˤ⡢ƵŪ˥ƥȤХɤʤ顢
		ǽŪ obj ̾ name <c_n> ˥Хɤ롣
		
		ξǤ⡢n-1ܤΥƥȾ name<n> Υ֥
		(Object 뤤 ƥ) ХɤƤ
		AlreadyBound 㳰ȯ롣
		
		\param name_list ֥Ȥդ̾ NameComponentꥹ
		\param obj Ϣդ Object
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		
		\exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
		\exception CannotProceed 餫ͳǽ³Ǥʤ
		\exception InvalidName  name ̾
		\exception AlreadyBound name <c_n>  Object Ǥ˥ХɤƤ롣
		
		\else
		
		\brief
		
		\endif
		"""
		if force == None :
			force = True

		try:
			self._rootContext.bind(name_list, obj)
		except CosNaming.NamingContext.NotFound:
			if force:
				self.bindRecursive(self._rootContext, name_list, obj)
			else:
				raise
		except CosNaming.NamingContext.CannotProceed, err:
			if force:
				self.bindRecursive(err.cxt, err.rest_of_name, obj)
			else:
				raise
		except CosNaming.NamingContext.AlreadyBound:
			self._rootContext.rebind(name_list, obj)


	def bindByString(self, string_name, obj, force=True):
		"""
		\if jp
		\brief Object  bind 
		
		Object  bind ݤͿ̾ʸɽǤ뤳Ȱʳϡbind()
		ƱǤ롣bind(toName(string_name), obj) 
		\param string_name ֥Ȥդ̾ʸɽ
		\param obj Ϣդ륪֥
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
		\exception CannotProceed 餫ͳǽ³Ǥʤ
		\exception InvalidName  name ̾
		\exception AlreadyBound name <n>  Object Ǥ˥ХɤƤ롣
		
		\else
		\brief
		\endif
		"""
		self.bind(self.toName(string_name), obj, force)


	def bindRecursive(self, context, name_list, obj):
		"""
		\if jp
		\brief ΥƥȤ bind ʤ Object  bind 
		
		context Ϳ줿 NamingContext Фơname_list ǻꤵ줿
		͡ॳݡͥ <c_1, ... c_(n-1)>  NamingContext Ȥ
		褷ʤ顢̾ <c_n> Ф obj  bind 롣
		⤷<c_1, ... c_(n-1)> б NamingContext ʤˤ
		 NamingContext Хɤ롣
		
		ǽŪ <c_1, c_2, ..., c_(n-1)> б NamingContext 
		ޤϲ褵줿ǡCosNaming::bind(<c_n>, object) ƤӽФ롣
		ΤȤǤ˥Хǥ󥰤¸ߤ AlreadyBound㳰ȯ롣
		
		ΥƥȤ褹ǡ褷褦Ȥ륳ƥȤ
		Ʊ̾ NamingContext ǤϤʤ Binding ¸ߤ硢
		CannotProceed 㳰ȯߤ롣
		\param context bind 򳫻Ϥ롡NamingContext
		\param name ֥Ȥդ̾Υ͡ॳݡͥ
		\param obj Ϣդ륪֥
		\exception CannotProceed <c_1, ..., c_(n-1)> б NamingContext 
		ΤҤȤĤǤ NamingContext ʳ object ˥Х
		Ƥꡢ³Ǥʤ
		\exception InvalidName ̾ name_list 
		\exception AlreadyBound name <c_n> ˤǤ˲餫 object Х
		Ƥ롣
		\else
		\brief
		\endif
		"""
		length = len(name_list)
		cxt = context
		for i in range(length):
			if i == length -1:
				try:
					cxt.bind(self.subName(name_list, i, i), obj)
				except CosNaming.NamingContext.AlreadyBound:
					cxt.rebind(self.subName(name_list, i, i), obj)
				return
			else:
				if self.objIsNamingContext(cxt):
					cxt = self.bindOrResolveContext(cxt,self.subName(name_list, i, i))
				else:
					raise CosNaming.NamingContext.CannotProceed(cxt, self.subName(name_list, i))
		return


	def rebind(self, name_list, obj, force=True):
		"""
		\if jp
		\brief Object  rebind 
		
		name_list ǻꤵ줿 Binding Ǥ¸ߤ bind() Ʊ
		Ǥ롣Хǥ󥰤Ǥ¸ߤˤϡХǥ󥰤
		֤롣
		\param name_list ֥Ȥդ̾ NameComponent
		\param obj Ϣդ륪֥
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\else
		\brief
		\endif
		"""
		if force == None:
			force = True
			
		try:
			self._rootContext.rebind(name_list, obj)

		except CosNaming.NamingContext.NotFound:
			if force:
				self.rebindRecursive(self._rootContext, name_list, obj)
			else:
				raise

		except CosNaming.NamingContext.CannotProceed, err:
			if force:
				self.rebindRecursive(err.cxt, err,rest_of_name, obj)
			else:
				raise
			
		return


	def rebindByString(self, string_name, obj, force=True):
		"""
		\if jp
		\brief Object  rebind 
		
		Object  rebind ݤͿ̾ʸɽǤ뤳Ȱʳ rebind()
		ƱǤ롣rebind(toName(string_name), obj) 
		\param string_name ֥Ȥդ̾ʸɽ
		\param obj Ϣդ륪֥
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
		\exception CannotProceed 餫ͳǽ³Ǥʤ
		\exception InvalidName  name ̾
		\else
		\brief
		\endif
		"""
		self.rebind(self.toName(string_name), obj, force)

		return


	def rebindRecursive(self, context, name_list, obj):
		"""
		\if jp
		\brief ΥƥȤ bind ʤ Object  rebind 
		
		name <c_n> ǻꤵ줿 NamingContext ⤷ Object Ǥ¸ߤ
		 bindRecursive() ƱǤ롣
		
		name <c_n> ǻꤵ줿Хǥ󥰤Ǥ¸ߤˤϡ
		Хǥ󥰤֤롣
		\param name_list ֥Ȥդ̾ʸɽ
		\param obj Ϣդ륪֥
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\exception CannotProceed ΥƥȤǤʤ
		\exception InvalidName Ϳ줿 name 
		\else
		\brief
		\endif
		"""
		length = len(name_list)
		for i in range(length):
			if i == length - 1:
				context.rebind(self.subName(name_list, i, i), obj)
				return
			else:
				if self.objIsNamingContext(context):
					try:
						context = context.bind_new_context(self.subName(name_list, i, i))
					except CosNaming.NamingContext.AlreadyBound:
						obj_ = context.resolve(self.subName(name_list, i, i))
						context = obj_._narrow(CosNaming.NamingContext)
				else:
					raise CosNaming.NamingContext.CannotProceed(context, self.subName(name_list, i))
		return


	def bindContext(self, name, name_cxt, force=True):
		"""
		\if jp
		\brief NamingContext  bind 
		
		bind 륪֥Ȥ NamingContext Ǥ뤳Ȥ bind() 
		ƱǤ롣
		\param name ֥Ȥդ̾ʸɽޤϡNameComponentΥꥹ
		\param name_cxt Ϣդ NamingContext
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\else
		\brief
		\endif
		"""
		if isinstance(name, basestring):
			self.bind(self.toName(name), name_cxt, force)
		else:
			self.bind(name, name_cxt, force)
		return


	def bindContextRecursive(self, context, name_list, name_cxt):
		"""
		\if jp
		\brief NamingContext  bind 
		
		bind 륪֥Ȥ NamingContext Ǥ뤳Ȥ
		bindRecursive() ƱǤ롣
		\param context bind 򳫻Ϥ롡NamingContext
		\param name_list ֥Ȥդ̾Υ͡ॳݡͥ
		\param name_cxt Ϣդ NamingContext
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\else
		\brief
		\endif
		"""
		self.bindRecursive(context, name_list, name_cxt)
		return


	def rebindContext(self, name, name_cxt, force=True):
		"""
		\if jp
		\brief NamingContext  rebind 
		
		name ǻꤵ줿ƥȤǤ¸ߤ bindContext() 
		ƱǤ롣
		Хǥ󥰤Ǥ¸ߤˤϡХǥ󥰤
		֤롣
		\param name ֥Ȥդ̾Υ͡ॳݡͥȤޤϡʸ
		\param name_cxt Ϣդ NamingContext
		\param force trueξ硢ΥƥȤŪ˥Хɤ
		\else
		\brief
		\endif
		"""
		if isinstance(name, basestring):
			self.rebind(self.toName(name), name_cxt, force)
		else:
			self.rebind(name, name_cxt, force)
		return


	def rebindContextRecursive(self, context, name_list, name_cxt):
		self.rebindRecursive(context, name_list, name_cxt)
		return


	def resolve(self, name):
		"""
		\if jp
		\brief Object  name 褹
		
		name  bind Ƥ륪֥ȻȤ֤
		͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
		
		CosNaming::resolve() ȤۤƱƯ򤹤뤬Ϳ줿
		͡ॵФΥ롼ȥƥȤФ resolve() ƤӽФ
		ۤʤ롣
		\param name 褹٤֥Ȥ̾Υ͡ॳݡͥ
		ޤϡ褹٤֥Ȥ̾ʸɽ
		\return 褵줿֥Ȼ
		\else
		\endif
		"""
		if isinstance(name, basestring):
			name_ = self.toName(name)
		else:
			name_ = name
			
		try:
			obj = self._rootContext.resolve(name_)
			return obj
		except CosNaming.NamingContext.NotFound, ex:
			return None


	def unbind(self, name):
		"""
		\if jp
		\brief ꤵ줿̾Υ֥Ȥ bind 
		
		name  bind Ƥ륪֥ȻȤ֤
		͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
		
		CosNaming::unbind() ȤۤƱƯ򤹤뤬Ϳ줿
		͡ॵФΥ롼ȥƥȤФ unbind() ƤӽФ
		ۤʤ롣
		\param name 褹٤֥Ȥ̾Υ͡ॳݡͥ
		ޤϡ褹٤֥Ȥ̾ʸɽ
		\return 褵줿֥Ȼ
		\else
		\endif
		"""
		if isinstance(name, basestring):
			name_ = self.toName(name)
		else:
			name_ = name

		self._rootContext.unbind(name_)
		return


	def newContext(self):
		"""
		\if jp
		\brief ƥȤ
		
		Ϳ줿͡ॵо줿 NamingContext ֤
		֤줿 NamingContext  bind Ƥʤ
		\return 줿 NamingContext
		\else
		\endif
		"""
		return self._rootContext.new_context()


	def bindNewContext(self, name, force=True):
		"""
		\if jp
		\brief ƥȤ bind 
		
		Ϳ줿 name ФƿƥȤХɤ롣
		줿NamingContext ϥ͡ॵо줿ΤǤ롣
		\param name NamingContextդ̾Υ͡ॳݡͥ
		ޤϡ褹٤֥Ȥ̾ʸɽ
		\return 줿 NamingContext
		\else
		\endif
		"""
		if force == None:
			force = True
			
		if isinstance(name, basestring):
			name_ = self.toName(name)
		else:
			name_ = name

		try:
			return self._rootContext.bind_new_context(name_)
		except CosNaming.NamingContext.NotFound:
			if force:
				self.bindRecursive(self._rootContext, name_, self.newContext())
			else:
				raise
		except CosNaming.NamingContext.CannotProceed, err:
			if force:
				self.bindRecursive(err.cxt, err.rest_of_name, self.newContext())
			else:
				raise
		return None


	def destroy(self, context):
		"""
		\if jp
		\brief NamingContext 󥢥ƥֲ
		
		context ǻꤵ줿 NamingContext 󥢥ƥֲ롣
		context ¾ΥƥȤХɤƤ NotEmpty 㳰
		ȯ롣
		\param context 󥢥ƥֲ NamingContext
		\else
		\brief Destroy the naming context
		
		Delete the specified naming context.
		any bindings should be <unbind> in which the given context is bound to
		some names before invoking <destroy> operation on it. 
		\param context NamingContext which is destroied.
		\endif
		"""
		context.destroy()


	def destroyRecursive(self, context):
		"""
		\if jp
		\brief NamingContext ƵŪ˲ä󥢥ƥֲ
		\else
		\brief Destroy the naming context recursively
		\endif
		"""
		cont = True
		bl = []
		bi = 0
		bl, bi = context.list(self._blLength)
		while cont:
			for i in range(len(bl)):
				if bl[i].binding_type == CosNaming.ncontext:
					obj = context.resolve(bl[i].binding_name)
					next_context = obj._narrow(CosNaming.NamingContext)

					self.destroyRecursive(next_context)
					context.unbind(bl[i].binding_name)
					next_context.destroy()
				elif bl[i].binding_type == CosNaming.nobject:
					context.unbind(bl[i].binding_name)
				else:
					assert(0)
			if CORBA.is_nil(bi):
				cont = False
			else:
				bi.next_n(self._blLength, bl)

		if not (CORBA.is_nil(bi)):
			bi.destroy()
		return


	def clearAll(self):
		"""
		\if jp
		\brief ٤Ƥ Binding 
		\else
		\brief Destroy all binding
		\endif
		"""
		self.destroyRecursive(self._rootContext)
		return


	def list(self, name_cxt, how_many, rbl, rbi):
		"""
		\if jp
		\brief Ϳ줿 NamingContext  Binding 
		\else
		\brief Get Binding on the NamingContextDestroy all binding
		\endif
		"""
		bl, bi = name_cxt.list(how_many)

		for i in bl:
			rbl.append(bl)

		rbi.append(bi)
	

    #============================================================
    # interface of NamingContext
    #============================================================

	def toString(self, name_list):
		"""
		\if jp
		\brief Ϳ줿 NameComponent ʸɽ֤
		\else
		\brief Get string representation of given NameComponent
		\endif
		"""
		if len(name_list) == 0:
			raise CosNaming.NamingContext.InvalidName

		slen = self.getNameLength(name_list)
		string_name = [""]
		self.nameToString(name_list, string_name, slen)

		return string_name


	def toName(self, sname):
		"""
		\if jp
		\brief Ϳ줿ʸɽ NameComponent ʬ򤹤
		\else
		\brief Get NameComponent from gien string name representation
		\endif
		"""
		if not sname:
			raise CosNaming.NamingContext.InvalidName

		string_name = sname
		name_comps = []

		nc_length = 0
		nc_length = self.split(string_name, "/", name_comps)
		if not (nc_length > 0):
			raise CosNaming.NamingContext.InvalidName

		name_list = [CosNaming.NameComponent("","") for i in range(nc_length)]

		for i in range(nc_length):
			pos = string.rfind(name_comps[i][0:],".")
			if pos == -1:
				name_list[i].id   = name_comps[i]
				name_list[i].kind = ""
			else:
				name_list[i].id   = name_comps[i][0:pos]
				name_list[i].kind = name_comps[i][(pos+1):]

		return name_list


	def toUrl(self, addr, string_name):
		"""
		\if jp
		\brief Ϳ줿 addre  string_name  URLɽ
		\else
		\brief Get URL representation from given addr and string_name
		\endif
		"""
		return self._rootContext.to_url(addr, string_name)


	def resolveStr(self, string_name):
		"""
		\if jp
		\brief Ϳ줿ʸɽ resolve ֥Ȥ֤
		\else
		\brief Resolve from name of string representation and get object 
		\endif
		"""
		return self.resolve(self.toName(string_name))


    #============================================================
    # Find functions
    #============================================================

	def bindOrResolve(self, context, name_list, obj):
		"""
		\if jp
		\brief ̾Хɤޤϲ褹
		\else
		\brief Bind of resolve the given name component
		\endif
		"""
		try:
			context.bind_context(name_list, obj)
			return obj
		except CosNaming.NamingContext.AlreadyBound:
			obj = context.resolve(name_list)
			return obj
		return CORBA.Object._nil


	def bindOrResolveContext(self, context, name_list, new_context=None):
		"""
		\if jp
		\brief ̾Хɤޤϲ褹
		\else
		\brief Bind of resolve the given name component
		\endif
		"""
		if new_context == None:
			new_cxt = self.newContext()
		else:
			new_cxt = new_context

		obj = self.bindOrResolve(context, name_list, new_cxt)
		return obj._narrow(CosNaming.NamingContext)


	def getNameServer(self):
		"""
		\if jp
		\brief ͡ॵФ̾
		\else
		\brief Get the name of naming server
		\endif
		"""
		return self._nameServer


	def getRootContext(self):
		"""
		\if jp
		\brief 롼ȥƥȤ
		\else
		\brief Get the root context
		\endif
		"""
		return self._rootContext


	def objIsNamingContext(self, obj):
		"""
		\if jp
		\brief ֥Ȥ͡ߥ󥰥ƥȤȽ̤
		\else
		\brief Whether the object is NamingContext
		\endif
		"""
		nc = obj._narrow(CosNaming.NamingContext)
		if CORBA.is_nil(nc):
			return False
		else:
			return True


	def nameIsNamingContext(self, name_list):
		"""
		\if jp
		\brief Ϳ줿̾͡ߥ󥰥ƥȤɤ
		\else
		\brief Whether the given name component is NamingContext
		\endif
		"""
		return self.objIsNamingContext(self.resolve(name_list))


	def subName(self, name_list, begin, end = None):
		"""
		\if jp
		\brief ͡ॳݡͥȤʬ֤
		\else
		\brief Get subset of given name component
		\endif
		"""
		if end == None or end < 0:
			end = len(name_list) - 1

		sub_len = end - (begin -1)
		objId = ""
		kind  = ""
		
		sub_name = []
		for i in range(sub_len):
			sub_name.append(name_list[begin + i])

		return sub_name


	def nameToString(self, name_list, string_name, slen):
		"""
		\if jp
		\brief ͡ॳݡͥȤʸɽ
		\else
		\brief Get string representation of name component
		\endif
		"""

		for i in range(len(name_list)):
			for id_ in name_list[i].id:
				if id_ == "/" or id_ == "." or id_ == "\\":
					string_name[0] += "\\"
				string_name[0] += id_

			if name_list[i].id == "" or name_list[i].kind != "":
				string_name[0] += "."

			for kind_ in name_list[i].kind:
				if kind_ == "/" or kind_ == "." or kind_ == "\\":
					string_name[0] += "\\"
				string_name[0] += kind_

			string_name[0] += "/"


	def getNameLength(self, name_list):
		"""
		\if jp
		\brief ͡ॳݡͥȤʸɽʸĹ
		\else
		\brief Get string length of the name component's string representation
		\endif
		"""
		slen = 0

		for i in range(len(name_list)):
			for id_ in name_list[i].id:
				if id_ == "/" or id_ == "." or id_ == "\\":
					slen += 1
				slen += 1
			if name_list[i].id == "" or name_list[i].kind == "":
				slen += 1

			for kind_ in name_list[i].kind:
				if kind_ == "/" or kind_ == "." or kind_ == "\\":
					slen += 1
				slen += 1

			slen += 1

		return slen


	def split(self, input, delimiter, results):
		"""
		\if jp
		\brief ʸʬ
		\else
		\brief Split of string
		\endif
		"""
		delim_size = len(delimiter)
		found_pos = begin_pos = pre_pos = substr_size = 0

		if input[0:delim_size] == delimiter:
			begin_pos = pre_pos = delim_size

		while 1:
			found_pos = string.find(input[begin_pos:],delimiter)
			
			if found_pos == -1:
				results.append(input[pre_pos:])
				break

			if found_pos > 0 and input[found_pos - 1] == "\\":
				begin_pos += found_pos + delim_size
			else:
				substr_size = found_pos + (begin_pos - pre_pos)
				if substr_size > 0:
					results.append(input[pre_pos:(pre_pos+substr_size)])
				begin_pos += found_pos + delim_size
				pre_pos   = begin_pos

		return len(results)
