#!/usr/bin/env python
# -*- Python -*-


"""
  \file OutPort.py
  \brief OutPort class
  \date $Date: 2007/09/19$
  \author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
 
  Copyright (C) 2006
      Noriaki Ando
      Task-intelligence Research Group,
      Intelligent Systems Research Institute,
      National Institute of
          Advanced Industrial Science and Technology (AIST), Japan
      All rights reserved.
"""


from omniORB import any

import OpenRTM

usec_per_sec = 1000000


import time
class Time:
	def __init__(self):
		global usec_per_sec
		tm = time.time()
		tm_f       = tm - int(tm)     # μФ
		self.sec   = int(tm - tm_f)   # μФ
		self.usec  = int(tm_f * usec_per_sec) # sec -> usec (micro second)



class OutPort(OpenRTM.OutPortBase):
	"""
	\if jp
	\class OutPort
	\brief OutPort 饹
	\else
	\endif
	"""


	def __init__(self, name, value, buffer_):
		"""
		\if jp
		\brief OutPort饹Υ󥹥ȥ饯
		\else
		\brief Constructor
		\param name Port name.
		\param value DataType.
		\param buffer_ Buffer object.
		\endif
		"""
		OpenRTM.OutPortBase.__init__(self, name)
		self._buffer         = buffer_
		self._value          = value
		self._timeoutTick    = 1000 # timeout tick: 1ms
		self._readBlock      = False
		self._readTimeout    = 0
		self._writeBlock     = False
		self._writeTimeout   = 0
		self._OnWrite        = None
		self._OnWriteConvert = None
		self._OnRead         = None
		self._OnReadConvert  = None
		self._OnOverflow     = None
		self._OnUnderflow    = None


	def isNew(self):
		return self._buffer.isNew()


	def write(self, value=None):
		"""
		\if jp
		\brief ǡ񤭹
		\else
		\brief Write data
		\endif
		"""
		if not value:
			value=self._value

		global usec_per_sec
		
		if self._OnWrite:
			self._OnWrite(value)

		timeout = self._writeTimeout

		tm_pre = Time()

		# blocking and timeout wait
		count = 0
		while self._writeBlock and self._buffer.isFull():
			if self._writeTimeout < 0:
				time.sleep(self._timeoutTick/usec_per_sec)
				continue
			
				
			# timeout wait
			tm_cur = Time()

			sec  = tm_cur.sec - tm_pre.sec
			usec = tm_cur.usec - tm_pre.usec

			timeout -= (sec * usec_per_sec + usec)

			if timeout < 0:
				break
			tm_pre = tm_cur
			time.sleep(self._timeoutTick/usec_per_sec)
			count += 1
      
		if self._buffer.isFull():
			if self._OnOverflow:
				self._OnOverflow(value)
			return False
      
		if not self._OnWriteConvert:
			self._buffer.put(value)
		else:
			self._buffer.put(self._OnWriteConvert(value))

		self.notify()
		return True


	def read(self, value):
		"""
		\if jp
		\brief ǡɤ߽Ф
		\else
		\brief Read data
		\endif
		"""
		if self._OnRead:
			self._OnRead()

		timeout = self._readTimeout
		tm_pre = Time()

		# blocking and timeout wait
		while self._readBlock and self._buffer.isEmpty():
			if self._readTimeout < 0:
				time.sleep(self._timeoutTick/usec_per_sec)
				continue

			# timeout wait
			tm_cur = Time()
			sec  = tm_cur.sec - tm_pre.sec
			usec = tm_cur.usec - tm_pre.usec
			
			timeout -= (sec * usec_per_sec + usec)
			if timeout < 0:
				break
			tm_pre = tm_cur
			time.sleep(self._timeoutTick/usec_per_sec)

		if self._buffer.isEmpty():
			if self._OnUnderflow:
				value[0] = self._OnUnderflow()
				return False
			else:
				return False

		if not self._OnReadConvert:
			value[0] = self._buffer.get()
			return True
		else:
			value[0] = self._OnReadConvert(self._buffer.get())
			return true

		# never comes here
		return False


	def setReadBlock(self, block):
		"""
		\if jp
		\brief read() Υ֥å֥å⡼ɤΥå
		\else
		\brief Set read() block mode
		\endif
		"""
		self._readBlock = block


	def setWriteBlock(self, block):
		"""
		\if jp
		\brief write() Υ֥å֥å⡼ɤΥå
		\else
		\brief Set read() block mode
		\endif
		"""
		self._writeBlock = block


	def setReadTimeout(self, timeout):
		"""
		\if jp
		\brief read() ΥॢȻ֤
		read() ΥॢȻ֤ usec ꤹ롣
		read() ϥ֥å⡼ɤǤʤФʤʤ
		\param timeout ॢȻ [usec]
		\else
		\brief Set read() timeout
		\endif
		"""
		self._readTimeout = timeout


	def setWriteTimeout(self, timeout):
		"""
		\if jp
		\brief write() ΥॢȻ֤
		
		write() ΥॢȻ֤ usec ꤹ롣
		write() ϥ֥å⡼ɤǤʤФʤʤ
		\param timeout ॢȻ [usec]
		\else
		\brief Set write() timeout
		\endif
		"""
		self._writeTimeout = timeout


	def setOnWrite(self, on_write):
		"""
		\if jp
		\brief OutWrite Хå
		\else
		\brief Set OnWrite callback
		\endif
		"""
		self._OnWrite = on_write


	def setOnWriteConvert(self, on_wconvert):
		"""
		\if jp
		\brief OutWriteConvert Хå
		\else
		\brief Set OnWriteConvert callback
		\endif
		"""
		self._OnWriteConvert = on_wconvert


	def setOnOverflow(self, on_overflow):
		"""
		\if jp
		\brief OutOverflow Хå
		\else
		\brief Set OnOverflow callback
		\endif
		"""
		self._OnOverflow = on_overflow


	def setOnRead(self, on_read):
		"""
		\if jp
		\brief OutRead Хå
		\else
		\brief Set OnRead callback
		\endif
		"""
		self._OnRead = on_read


	def setOnReadConvert(self, on_rconvert):
		"""
		\if jp
		\brief OutReadConvert Хå
		\else
		\brief Set OnReadConvert callback
		\endif
		"""
		self._OnReadConvert = on_rconvert


	def setOnUnderflow(self, on_underflow):
		"""
		\if jp
		\brief OutUnderflow Хå
		\else
		\brief Set OnUnderflow callback
		\endif
		"""
		self._OnUnderflow = on_underflow


	def getPortDataType(self):
		"""
		\brief ǡΤΥ᥽å
		ǡη̾뤿ᡢInPortCorbaProviderƤФ롣
		added by kurihara
		"""
		val = any.to_any(self._value)
		return str(val.typecode().name())
